#include "linux_socket.h"
#include <cstddef>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>

namespace Bk::Net {
	LinuxSocket::LinuxSocket(int id, struct sockaddr_in client_addr, IpVersion ver, IpProtocol proto)
	: id(id), ip_proto(proto), addr(client_addr)
	{
		char* myIP = inet_ntoa(addr.sin_addr);
		ip_addr = IpAddress(std::string(myIP), ver);
	}

	LinuxSocket::LinuxSocket(IpAddress ip, int port, IpProtocol proto)
	: ip_addr(ip), ip_proto(proto)
	{
		//LinuxSocket creation step
		if ((id = socket((int)ip_addr.version, (int)ip_proto, 0)) < 0) 
		{
	        BK_CORE_ERROR("Socket failed");
	        exit(EXIT_FAILURE);
	    }
	    addr.sin_addr = ip_addr.get_data();
		addr.sin_family = (int)ip_addr.version;
    	addr.sin_port = htons(port);
	}

	LinuxSocket::~LinuxSocket()
	{
		if(id > 0) close(id);
	}

	bool LinuxSocket::init()
	{
	    //Binding step
		int status;
	    if ((status = bind(id, (struct sockaddr*)&addr, sizeof(addr)) < 0)) 
	    {
			BK_CORE_ERROR("Binding failed");
	      	return false;
	    }
	    return true;
	}

	bool LinuxSocket::start(int cpt_conn)
	{
	    //Listening step
	    if (listen(id, cpt_conn) < 0) 
	    {
			BK_CORE_ERROR("Listening failed");
	    	return false;
	    }
	    return true;
	}

	Socket* LinuxSocket::ack()
	{
		struct sockaddr_in client_addr;
		socklen_t addrlen = sizeof(client_addr);
		return new LinuxSocket(accept(id, (struct sockaddr*)&client_addr, &addrlen), client_addr, ip_addr.version, ip_proto);
	}

	bool LinuxSocket::conn()
	{
		if (connect(id, (struct sockaddr*)&addr, sizeof(addr)) != 0) 
		{
			BK_CORE_ERROR("Connection failed");
	        return false;
    	}
    	return true;
	}

	bool LinuxSocket::isConnected()
	{
		char data = 0;
		int read_size = recv(id, &data, 1, MSG_PEEK);
		return (bool)read_size;
	}

	void LinuxSocket::emit(std::vector<char> packet)
	{
		size_t size = send(id, packet.data(), packet.size(), 0);
	}

	std::vector<char> LinuxSocket::obtain(int size)
	{
		if(size > this->size)
		{
			delete [] buffer;
			buffer = new char[size];
			this->size = size;
		}
		int read_size = recv(id, buffer, this->size - 1, 0);
		if(read_size == -1) return std::vector<char>(0);
		std::vector<char> packet(buffer, buffer + read_size);
		return packet;
	}
	
	Socket* Socket::create(IpAddress ip, int port, IpProtocol proto)
	{
		return new LinuxSocket(ip, port, proto);
	}
}